#!/bin/bash

########################################global config#################################################
script_name=$(basename $0)
version_path=/etc/cpeagent/
logfile="/var/log/update.log"
upgrade_path=/etc/cpeagent/upgrade/${script_name%.*}
patch_name=${script_name%.*}
target_name=${upgrade_path}/${script_name%.*}.tar.gz
ha_conf_file=/etc/cpeagent/ha.conf
vrrp_state_file=/var/run/keepalived.state
cpeagent_conf_file=/etc/cpeagent/cpeagent.conf

# success 
ret_success=0
ret_success_restart=1
ret_success_reboot=2
# fail and need rollback
ret_fail_roolback=6
# fail but no rollback
ret_fail_check_version=7
ret_fail_ha_interface=8
ret_fail_check_ha_status=9
ret_fail_para=10
ret_fail_init_ha_role=11
ret_fail_ha_enable=12
ret_fail_ha_vrrp=13
ret_fail_miss=14
ret_fail_install=15
ret_fail_opration=16
ret_fail_others=17

# 万达版本集合，当前仅万达可升级测试
# 总部集群      当前版本5.3.0.250    LW7009-8xGE-8x10GE
# 二层组网      当前版本5.0.0.230    LW7009-10×GE-4×10GE
# 财务          当前版本5.0.0.230    LW3007V3-2x10GE
#               当前版本5.0.0.230    LW7009-8xGE-4x10GE-GM
#               当前版本5.0.0.230    LW3003V3
#               当前版本5.0.0.230    2508
# 分支          当前版本5.0.0.230    2509+2309
#               当前版本5.0.0.230    2508+2308
#灰度升级的版本：5.0.0.316、5.3.0.362、5.3.0.330
#要升级的版本：5.5.0.611
SUPPORT_UPDRPMVERSION_CHECK=1
SUPPORT_UPDRPMVERSION_LIST="
5.3.0.250
5.0.0.230
5.0.0.316
5.3.0.362
5.3.0.330
5.5.0.611
"

print_prefix=""
upgrade_result_query_count=3
upgrade_result_query_time1_wait=1
upgrade_result_query_time2_wait=$((upgrade_result_query_time1_wait * 2))
upgrade_result_query_time3_wait=$((upgrade_result_query_time2_wait * 2))
# sshpass configuration
ssh_port=20022
ssh_connection_timeout=3
scp_connection_timeout=10
ssh_strict_hostkey_check="no"
sshpass_cmd="/bin/appex/ha/sshpass"
ssh_options="-x ssh -p $ssh_port -o StrictHostKeyChecking=$ssh_strict_hostkey_check -o ConnectTimeout=$ssh_connection_timeout"
scp_options="-x scp -P $ssh_port -o StrictHostKeyChecking=$ssh_strict_hostkey_check -o ConnectTimeout=$scp_connection_timeout"

# ha enable state array, 0 for local, 1 for peer
# value 0  disable 
#       1  enable
declare -a ha_enable_state
ha_enable_state[0]=0
ha_enable_state[1]=0

# real-time vrrp state array, 0 for local, 1 for peer
# value 0  default
#       1  master
#       2  backup
declare -a vrrp_state
vrrp_state[0]=0
vrrp_state[1]=0

# initial HA role array, 0 for local, 1 for peer
# value 0  default
#       1  master
#       2  backup
declare -a init_ha_role
init_ha_role[0]=0
init_ha_role[1]=0

# current available ha interface and current peer ip
current_ha_interface=""
peer_ha_interface_ip=""



########################################  function  ####################################################
log()
{
    echo `date` "upgrade:" $1 >> $logfile
}

log_exit()
{
    echo `date` "upgrade:" $1 >> $logfile
    exit $2
}

check_RPM_Version()
{
    if [ $SUPPORT_UPDRPMVERSION_CHECK -eq 0 ] ;then
        return $ret_success
    fi
    curVersion=$(cat /etc/cpeagent/RPM_version)
    for str in ${SUPPORT_UPDRPMVERSION_LIST}; do
        if [ "${curVersion}" == "${str}" ]; then
            return $ret_success
        fi
    done
    return $ret_fail_check_version
}

# install_rpm_package <local/peer> <rpm_path> <rpm_ver>
install_single_rpm_package()
{
    if [ "$1" == "local" ]; then
        rpm_query_cmd="rpm -q $3 > /dev/null 2>&1"
        check_rpm_exist_cmd="[ ! -f $2 ]"
        rpm_install_cmd="rpm -ivh $2 --nodeps >> $logfile 2>&1"
    elif [ "$1" == "peer" ]; then
        rpm_query_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip rpm -q $3 > /dev/null 2>&1"
        check_rpm_exist_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip [ ! -f $2 ]"
        rpm_install_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip rpm -ivh $2 --nodeps >> $logfile 2>&1"
    else
        log_exit "$print_prefix install_single_rpm_package para error" $ret_fail_para
    fi

    # return if already installed 
    eval $rpm_query_cmd
    if [ $? -eq 0 ]; then
        log "$print_prefix $1 $2 has been installed already, no need install again"
        return $ret_success
    fi

    # exit if rpm package is not exist
    eval $check_rpm_exist_cmd && log_exit "$1 missing $1" $ret_fail_miss_rpm

    # install rpm package
    eval $rpm_install_cmd || log_exit "$1 rpm install $1 failed" $ret_fail_install
}

# install onig && jq for master and backup
install_dependency_rpm()
{
    install_single_rpm_package $1 ${upgrade_path}/packages/oniguruma-6.8.2-2.el7.x86_64.rpm oniguruma-6.8.2-2.el7.x86_64
    install_single_rpm_package $1 ${upgrade_path}/packages/jq-1.6-2.el7.x86_64.rpm jq-1.6-2.el7.x86_64
}

send_patch_to_backup()
{
    # backup need mkdir upgrade_path
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip mkdir -p $upgrade_path
    if [ $? -ne 0 ]; then
        log "$print_prefix mkdir $upgrade_path failed"
        return $ret_fail_opration
    fi

    # scp patch to backup
    $sshpass_cmd $scp_options $target_name root@$peer_ha_interface_ip:$upgrade_path
    if [ $? -ne 0 ]; then
        log "$print_prefix scp patch failed"
        return $ret_fail_opration
    fi

    # tar patch 
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip "cd $upgrade_path && tar -xzvf $target_name > /dev/null 2>&1"
    if [ $? -ne 0 ]; then
        log "$print_prefix tar patch failed"
        return $ret_fail_opration
    fi

    return $ret_success
}

check_backup_install_result()
{
    # check LW_update.sh has been replaced or not
    orgi_md5=$($sshpass_cmd $ssh_options root@$peer_ha_interface_ip md5sum $upgrade_path/script/LW_update.sh | awk '{print $1}')
    new_md5=$($sshpass_cmd $ssh_options root@$peer_ha_interface_ip md5sum /bin/appex/LW_update.sh | awk '{print $1}')

    if [[ "$orgi_md5" == "$new_md5" ]]; then
        return $ret_success
    fi
    return $ret_fail_install
}

# install sshpass for master and backup
install_sshpass()
{
    if [ "$1" == "local" ]; then
        cp_cmd="\cp -rf ${upgrade_path}/sshpass/sshpass /bin/appex/ha/sshpass"
        chmod_cmd="chmod 755 /bin/appex/ha/sshpass"
    elif [ "$1" == "peer" ]; then
        cp_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip \cp -rf ${upgrade_path}/sshpass/sshpass /bin/appex/ha/sshpass"
        chmod_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip chmod 755 /bin/appex/ha/sshpass"
    else
        log_exit "$print_prefix install_sshpass para error" $ret_fail_para
    fi

    eval $cp_cmd || log_exit "$print_prefix $1 cp sshpass failed" $ret_fail_opration
    eval $chmod_cmd || log_exit "$print_prefix $1 chmod sshpass failed" $ret_fail_opration
}

# install LW_update.sh for master and backup
install_lw_update_script()
{
    if [ $1 == "local" ]; then
        bak_cmd="\cp -rf /bin/appex/LW_update.sh /bin/appex/LW_update.sh.bak"
        cp_cmd="\cp -rf ${upgrade_path}/script/LW_update.sh /bin/appex/LW_update.sh"
    elif [ $1 == "peer" ]; then
        bak_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip \cp -rf /bin/appex/LW_update.sh /bin/appex/LW_update.sh.bak"
        cp_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip \cp -rf ${upgrade_path}/script/LW_update.sh /bin/appex/LW_update.sh"
    else
        log_exit "$print_prefix install_lw_update_script para error" $ret_fail_para
    fi

    eval $bak_cmd || log_exit "$1 backup LW_update.sh failed" $ret_fail_roolback
    eval $cp_cmd || log_exit "$1 cp LW_update.sh failed" $ret_fail_roolback

    # write file with upgrade result
    if [ $1 == "peer" ]; then
        $sshpass_cmd $ssh_options root@$peer_ha_interface_ip "echo ${patch_name##*-} > ${version_path}/PATCH-CUSTOM-VERSION"
    fi
}

# get HA status for local and peer
# get_HA_status <local/peer>
get_HA_status()
{
    if [ $1 == "local" ]; then
        check_ha_conf_cmd="[ -f $ha_conf_file ]"
        ha_enable_cmd="cat $ha_conf_file | jq -r '.haEnable'"
        check_vrrp_state_cmd="[ -f $vrrp_state_file ]"
        vrrp_state_cmd="cat $vrrp_state_file"
        check_cpagent_conf_cmd="[ -f $cpeagent_conf_file ]"
        init_ha_role_cmd="cat $cpeagent_conf_file | grep ha="
        index=0
    elif [ $1 == "peer" ]; then
        check_ha_conf_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip [ -f $ha_conf_file ]"
        ha_enable_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip cat $ha_conf_file | jq -r '.haEnable'"
        check_vrrp_state_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip [ -f $vrrp_state_file ]"
        vrrp_state_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip cat $vrrp_state_file"
        check_cpagent_conf_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip [ -f $cpeagent_conf_file ]"
        init_ha_role_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip cat $cpeagent_conf_file | grep ha="
        index=1
    else
        log "$print_prefix get_HA_status para error" 
        return $ret_fail_para
    fi

    # check initial ha role
    eval $check_cpagent_conf_cmd
    if [ $? -ne 0 ]; then
        log "$print_prefix $1 missing $cpeagent_conf_file"
        return $ret_fail_miss
    fi
    init_role=$(eval $init_ha_role_cmd)
    if [ "${init_role}" == "ha=1" ]; then
        init_ha_role[$index]=1
    elif [ "${init_role}" == "ha=2" ]; then
        init_ha_role[$index]=2
    elif [ "${init_role}" == "ha=0" ]; then
        init_ha_role[$index]=0
        log "$print_prefix $1 ha=0 which means ha is disable" 
        return $ret_success
    else
        log "$print_prefix $1 ha init role $init_role error" 
        return $ret_fail_init_ha_role
    fi

    # check ha is enable
    eval $check_ha_conf_cmd 
    if [ $? -ne 0 ]; then
        log "$print_prefix $1 missing $ha_conf_file"
        return $ret_fail_miss
    fi
    enable=$(eval $ha_enable_cmd)
    if [ "${enable}" == "true" ]; then
        ha_enable_state[$index]=1
    elif [ "${enable}" == "false" ]; then
        ha_enable_state[$index]=0
    else
        log "$print_prefix $1 ha enable status $enable error" 
        return $ret_fail_ha_enable
    fi

    # check state
    eval $check_vrrp_state_cmd 
    if [ $? -ne 0 ]; then
        log "$print_prefix $1 missing $vrrp_state_file"
        return $ret_fail_miss
    fi 
    state=$(eval $vrrp_state_cmd)
    if [[ "$state" =~ "MASTER" ]]; then
        vrrp_state[$index]=1
    elif [[ "$state" =~ "BACKUP" ]]; then
        vrrp_state[$index]=2
    else
        log "$print_prefix $1 vrrp status $state error" 
        return $ret_fail_ha_vrrp
    fi
    return $ret_success
}

# check local and peer HA status based on the real-time HA role
# ha is enable and MASTER & BACKUP exist
check_HA_vrrp_status()
{
    if [ ${ha_enable_state[0]} -eq 0 ];then
        return 0
    fi
    if [ ${ha_enable_state[0]} == 1 ] && [ ${ha_enable_state[1]} == 1 ]; then
        if [ ${vrrp_state[0]} == 1 ] && [ ${vrrp_state[1]} == 2 ]; then
            return 0
        elif [ ${vrrp_state[0]} == 2 ] && [ ${vrrp_state[1]} == 1 ]; then
            return 0
        fi
    fi
    return $ret_fail_check_ha_status
}

# get peer ip based on the initial HA role, not the real-time HA role
# masterip = 169.254.201.0 + index << 8 + 2
# backupip = 169.254.201.0 + index << 8 + 3
get_peer_ip()
{
    if [ ${init_ha_role[0]} -eq 1 ] ;then
        peerip="169.254.$((201 + $1)).3"
    else
        peerip="169.254.$((201 + $1)).2"
    fi
    echo "$peerip"
}

# test_available eth5 169.254.201.3
test_available()
{
    if [ $# -ne 2 ]; then
        return $ret_fail_para
    fi

    ping -c 1 -W 1 -I "$1" "$2" > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        return 0
    else
        return $ret_fail_ha_interface
    fi
}

get_valid_ha_interface()
{
    # check HA interface numbers
    nums=$(cat "$ha_conf_file" | jq '.haInterface | length')
    if [ "$nums" -le 0 ] ; then
        log "$print_prefix ha interface nums is zero"
        return $ret_fail_ha_interface
    fi

    for ((i = 0; i < $nums; i++ )); do
        ha_interface=$(cat "$ha_conf_file" | jq -r ".haInterface[$i]")
        peerip=$(get_peer_ip $i)
        test_available "$ha_interface" "$peerip"
        if [ $? -eq 0 ] ; then
            current_ha_interface="$ha_interface"
            peer_ha_interface_ip="$peerip"
            return $ret_success
        fi
    done
    return $ret_fail_ha_interface
}

# update LW_update.sh script for HA master and backup device
install()
{
    # check rpm version
    check_RPM_Version
    if [ $? -ne 0 ]; then
        log_exit "$print_prefix The PATCH package is not suitable for the current RPM version!" $ret_fail_check_version
    fi
    log "$print_prefix RPM version check ok!"

    # local install jq && sshpass
    install_sshpass "local"
    install_dependency_rpm "local"
    log "$print_prefix local install sshpass && jq ok!"

    # get local HA status
    get_HA_status "local"
    ret=$?
    if [ $ret -ne 0 ]; then
        log_exit "$print_prefix get local HA status error, ret=$ret" $ret
    fi
    log "$print_prefix local ha status ok, init_role:${init_ha_role[0]} enable:${ha_enable_state[0]} vrrp_state:${vrrp_state[0]}"

    # if ha is enable
    if [ ${init_ha_role[0]} -ne 0 ]; then
        get_valid_ha_interface
        if [ $? -ne 0 ]; then
            log_exit "$print_prefix can not get a valid ha interface" $ret_fail_ha_interface
        fi
        log "$print_prefix ha interface ok, interface:$current_ha_interface peer ip:$peer_ha_interface_ip is available"

        # if local is master, first，send patch to back to install jq && sshpass, otherwise can not get peer HA status
        send_patch_to_backup
        if [ $? -ne 0 ]; then
            log_exit "$print_prefix send patch to backup error" $ret_fail_install
        fi
        log "$print_prefix local is Matser, send rpm to peer ok!"

        # peer install jq && sshpass
        install_sshpass "peer"
        install_dependency_rpm "peer"
        log "$print_prefix peer install sshpass && jq ok!"

        # get peer HA status
        get_HA_status "peer"
        ret=$?
        if [ $ret -ne 0 ]; then
            log_exit "$print_prefix get peer HA status error $ret" $ret
        fi
        log "$print_prefix peer ha status ok, init_role:${init_ha_role[1]} enable:${ha_enable_state[1]} vrrp_state:${vrrp_state[1]}"

        check_HA_vrrp_status
        if [ $? -ne 0 ]; then
            log_exit "check HA state failed, local is ${vrrp_state[0]}, peer is ${vrrp_state[1]}, please check it first" $ret_fail_check_ha_status
        fi
        log "$print_prefix ha status check ok, local is ${vrrp_state[0]}, peer is ${vrrp_state[1]}"
    else
        log "$print_prefix ha is disable, upgrade as a single device"
    fi

    # this is common update LW_update.sh logic
    # if ha is enable
    if [ ${init_ha_role[0]} -ne 0 ]; then
        # peer install lw_update.sh first
        install_lw_update_script "peer"
        log "$print_prefix peer install lw_update ok!"

        # check peer install result
        for ((i = 1; i <= $upgrade_result_query_count; i++ )); do
            check_backup_install_result
            if [ $? -ne 0 ]; then
                log "$print_prefix $i time check peer install error"
                wait_time_var="upgrade_result_query_time${i}_wait"
                sleep ${!wait_time_var}
            else
                log "$print_prefix $patch_name peer install success"
                break
            fi
        done
        if [ $i -eq $((upgrade_result_query_count + 1)) ] ; then 
            log_exit "$print_prefix check peer install error" $ret_fail_roolback
        fi
    fi

    install_lw_update_script "local"
    log "$print_prefix local install lw_update ok!"
    log_exit "$print_prefix $patch_name install success!" $ret_success
}

rollback()
{
    if [ $1 == "local" ]; then
        rollback_cmd="[ -f /bin/appex/LW_update.sh.bak ] && \cp -rf /bin/appex/LW_update.sh.bak /bin/appex/LW_update.sh"
    elif [ $1 == "peer" ]; then
        rollback_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip [ -f /bin/appex/LW_update.sh.bak ] && \cp -rf /bin/appex/LW_update.sh.bak /bin/appex/LW_update.sh"
    else
        log_exit "$print_prefix rollback para error" $ret_fail_para
    fi
    eval $rollback_cmd || log_exit "$print_prefix $1 rollback failed" $ret_fail_roolback 
}

# rollback LW_update.sh script, only need to rollback LW_update.sh, jq & sshpass do not affect anything.
uninstall()
{
    # get local HA status
    get_HA_status "local"
    ret=$?
    if [ $ret -ne 0 ]; then
        log_exit "$print_prefix get local HA status error, ret=$ret" $ret
    fi
    # if ha is enable
    if [ ${init_ha_role[0]} -ne 0 ]; then
        get_valid_ha_interface
        if [ $? -ne 0 ]; then
            log_exit "$print_prefix can not get a valid ha interface" $ret_fail_ha_interface
        fi

        rollback "peer"
    fi
    # rollback, only need to rollback LW_update.sh, jq & sshpass do not affect anything.
    rollback "local"
    log_exit "$print_prefix $patch_name rollback success!" $ret_success 
}

if [ "$1" == "install" ];then
    install
elif [ "$1" == "uninstall" ]; then
    uninstall
else
    log_exit "invalid args" $ret_fail_para
fi
