#!/bin/sh
#version 1.1
set -u

PRODUCT=CPE
PRODUCT_LOWCASE=cpe
upgrade_path=/etc/"$PRODUCT_LOWCASE"agent/upgrade
version_path=/etc/"$PRODUCT_LOWCASE"agent/
conf_file=/etc/${PRODUCT_LOWCASE}agent/${PRODUCT_LOWCASE}agent.conf
bak_conf_file=/etc/${PRODUCT_LOWCASE}agent/${PRODUCT_LOWCASE}agent.conf.upgradebak

logfile="/var/log/update.log"
target="version"
patch_target_type=""
target_name=""
result_id=""

ret_success=0
ret_success_restart=1
ret_success_reboot=2
ret_fail_roolback=6
ret_fail_check_version=7
ret_fail_ha_interface=8
ret_fail_check_ha_status=9
ret_fail_para=10
ret_fail_init_ha_role=11
ret_fail_ha_enable=12
ret_fail_ha_vrrp=13
ret_fail_miss=14
ret_fail_install_rpm=15
ret_fail_opration=16
ret_fail_others=17

print_prefix=""
ha_conf_file=/etc/cpeagent/ha.conf
vrrp_state_file=/var/run/keepalived.state
cpeagent_conf_file=/etc/cpeagent/cpeagent.conf
upgrade_result_query_count=8
upgrade_result_query_time1_wait=4
upgrade_result_query_time2_wait=$((upgrade_result_query_time1_wait * 2))
upgrade_result_query_time3_wait=$((upgrade_result_query_time2_wait * 2))
upgrade_result_query_time4_wait=$((upgrade_result_query_time3_wait * 2))
upgrade_result_query_time5_wait=$((upgrade_result_query_time4_wait * 2))
upgrade_result_query_time6_wait=$((upgrade_result_query_time5_wait * 2))
upgrade_result_query_time7_wait=$((upgrade_result_query_time6_wait * 2))
upgrade_result_query_time8_wait=$((upgrade_result_query_time7_wait * 2))
ssh_port=20022
ssh_connection_timeout=3
scp_connection_timeout=10
ssh_strict_hostkey_check="no"
sshpass_cmd="/bin/appex/ha/sshpass"
ssh_options="-x ssh -p $ssh_port -o StrictHostKeyChecking=$ssh_strict_hostkey_check -o ConnectTimeout=$ssh_connection_timeout"
scp_options="-x scp -P $ssh_port -o StrictHostKeyChecking=$ssh_strict_hostkey_check -o ConnectTimeout=$scp_connection_timeout"
declare -a ha_enable_state
ha_enable_state[0]=0
ha_enable_state[1]=0
declare -a vrrp_state
vrrp_state[0]=0
vrrp_state[1]=0
declare -a init_ha_role
init_ha_role[0]=0
init_ha_role[1]=0
current_ha_interface=""
peer_ha_interface_ip=""


log()
{
    echo `date` "upgrade:" $@ >> $logfile
}


# get HA status for local and peer
# get_HA_status <local/peer>
get_HA_status()
{
    if [ $1 == "local" ]; then
        check_ha_conf_cmd="[ -f $ha_conf_file ]"
        ha_enable_cmd="cat $ha_conf_file | jq -r '.haEnable'"
        check_vrrp_state_cmd="[ -f $vrrp_state_file ]"
        vrrp_state_cmd="cat $vrrp_state_file"
        check_cpagent_conf_cmd="[ -f $cpeagent_conf_file ]"
        init_ha_role_cmd="cat $cpeagent_conf_file | grep ha="
        index=0
    elif [ $1 == "peer" ]; then
        check_ha_conf_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip [ -f $ha_conf_file ]"
        ha_enable_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip cat $ha_conf_file | jq -r '.haEnable'"
        check_vrrp_state_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip [ -f $vrrp_state_file ]"
        vrrp_state_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip cat $vrrp_state_file"
        check_cpagent_conf_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip [ -f $cpeagent_conf_file ]"
        init_ha_role_cmd="$sshpass_cmd $ssh_options root@$peer_ha_interface_ip cat $cpeagent_conf_file | grep ha="
        index=1
    else
        log "$print_prefix get_HA_status para error" 
        return $ret_fail_para
    fi

    # check initial ha role
    eval $check_cpagent_conf_cmd
    if [ $? -ne 0 ]; then
        log "$print_prefix $1 missing $cpeagent_conf_file"
        return $ret_fail_miss
    fi
    init_role=$(eval $init_ha_role_cmd)
    if [ "${init_role}" == "ha=1" ]; then
        init_ha_role[$index]=1
    elif [ "${init_role}" == "ha=2" ]; then
        init_ha_role[$index]=2
    elif [ "${init_role}" == "ha=0" ]; then
        init_ha_role[$index]=0
        log "$print_prefix $1 ha=0 which means ha is disable" 
        return $ret_success
    else
        log "$print_prefix $1 ha init role $init_role error" 
        return $ret_fail_init_ha_role
    fi

    # check ha is enable
    eval $check_ha_conf_cmd 
    if [ $? -ne 0 ]; then
        log "$print_prefix $1 missing $ha_conf_file"
        return $ret_fail_miss
    fi
    enable=$(eval $ha_enable_cmd)
    if [ "${enable}" == "true" ]; then
        ha_enable_state[$index]=1
    elif [ "${enable}" == "false" ]; then
        ha_enable_state[$index]=0
    else
        log "$print_prefix $1 ha enable status $enable error" 
        return $ret_fail_ha_enable
    fi

    # check state
    eval $check_vrrp_state_cmd 
    if [ $? -ne 0 ]; then
        log "$print_prefix $1 missing $vrrp_state_file"
        return $ret_fail_miss
    fi 
    state=$(eval $vrrp_state_cmd)
    if [[ "$state" =~ "MASTER" ]]; then
        vrrp_state[$index]=1
    elif [[ "$state" =~ "BACKUP" ]]; then
        vrrp_state[$index]=2
    else
        log "$print_prefix $1 vrrp status $state error" 
        return $ret_fail_ha_vrrp
    fi
    return $ret_success
}

# get peer ip based on the initial HA role, not the real-time HA role
# masterip = 169.254.201.0 + index << 8 + 2
# backupip = 169.254.201.0 + index << 8 + 3
get_peer_ip()
{
    if [ ${init_ha_role[0]} -eq 1 ] ;then
        peerip="169.254.$((201 + $1)).3"
    else
        peerip="169.254.$((201 + $1)).2"
    fi
    echo "$peerip"
}

# test_available eth5 169.254.201.3
test_available()
{
    if [ $# -ne 2 ]; then
        return $ret_fail_para
    fi

    ping -c 1 -W 1 -I "$1" "$2" > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        return 0
    else
        return $ret_fail_ha_interface
    fi
}

get_valid_ha_interface()
{
    # check HA interface numbers
    nums=$(cat "$ha_conf_file" | jq '.haInterface | length')
    if [ "$nums" -le 0 ] ; then
        log "$print_prefix ha interface nums is zero"
        return $ret_fail_ha_interface
    fi

    for ((i = 0; i < $nums; i++ )); do
        ha_interface=$(cat "$ha_conf_file" | jq -r ".haInterface[$i]")
        peerip=$(get_peer_ip $i)
        test_available "$ha_interface" "$peerip"
        if [ $? -eq 0 ] ; then
            current_ha_interface="$ha_interface"
            peer_ha_interface_ip="$peerip"
            return $ret_success
        fi
    done
    return $ret_fail_ha_interface
}

# check local and peer HA status based on the real-time HA role
# ha is enable and MASTER & BACKUP exist
check_HA_vrrp_status()
{
    if [ ${ha_enable_state[0]} -eq 0 ];then
        return 0
    fi
    if [ ${ha_enable_state[0]} == 1 ] && [ ${ha_enable_state[1]} == 1 ]; then
        if [ ${vrrp_state[0]} == 1 ] && [ ${vrrp_state[1]} == 2 ]; then
            return 0
        elif [ ${vrrp_state[0]} == 2 ] && [ ${vrrp_state[1]} == 1 ]; then
            return 0
        fi
    fi
    return $ret_fail_check_ha_status
}

send_rpm_to_backup_install()
{
    # backup need mkdir upgrade_path
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip mkdir -p $upgrade_path
    if [ $? -ne 0 ]; then
        log "$print_prefix mkdir $upgrade_path failed"
        return $ret_fail_opration
    fi

    # scp rpm to backup
    $sshpass_cmd $scp_options ${upgrade_path}/${target_name} root@$peer_ha_interface_ip:$upgrade_path
    if [ $? -ne 0 ]; then
        log "$print_prefix scp rpm failed"
        return $ret_fail_opration
    fi

    # like orch's upgrade task, use LW_update.sh to install peer device
    # NOTE: NOT use /bin/appex/LW_update.sh directly, becase it will call path shell which will update /bin/appex/LW_update.sh, this recursive calling may be except
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip "cp -f /bin/appex/LW_update.sh $upgrade_path/LW_update.sh"
    if [ $? -ne 0 ]; then
        log "$print_prefix cp LW_update.sh failed"
        return $ret_fail_opration
    fi
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip "bash $upgrade_path/LW_update.sh -t=version -tn=$target_name -r=$result_id > /dev/null 2>&1 &"
    if [ $? -ne 0 ]; then
        log "$print_prefix rpm install $PRODUCT failed"
        return $ret_fail_install_rpm
    fi
    return $ret_success
}

send_patch_to_backup_install()
{
    # backup need mkdir upgrade_path
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip mkdir -p $upgrade_path/$target_name
    if [ $? -ne 0 ]; then
        log "$print_prefix mkdir $upgrade_path/$target_name failed"
        return $ret_fail_opration
    fi

    # scp patch to backup
    $sshpass_cmd $scp_options ${upgrade_path}/${target_name}/${target_name}.tar.gz root@$peer_ha_interface_ip:$upgrade_path/$target_name
    if [ $? -ne 0 ]; then
        log "$print_prefix scp patch failed"
        return $ret_fail_opration
    fi

    # tar patch
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip "cd ${upgrade_path}/${target_name} && tar -xzvf ${target_name}.tar.gz"
    if [ $? -ne 0 ]; then
        log "$print_prefix tar patch failed"
        return $ret_fail_opration
    fi

    # like orch's upgrade task, use LW_update.sh to install peer device
    # NOTE: NOT use /bin/appex/LW_update.sh directly, becase it will call path shell which will update /bin/appex/LW_update.sh, this recursive calling may be except
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip "cp -f /bin/appex/LW_update.sh $upgrade_path/$target_name/LW_update.sh"
    if [ $? -ne 0 ]; then
        log "$print_prefix cp LW_update.sh failed"
        return $ret_fail_opration
    fi
    $sshpass_cmd $ssh_options root@$peer_ha_interface_ip "bash $upgrade_path/$target_name/LW_update.sh -t=patch -tn=$target_name -r=$result_id > /dev/null 2>&1 &"
    if [ $? -ne 0 ]; then
        log "$print_prefix rpm install $PRODUCT failed"
        return $ret_fail_install_rpm
    fi
    return $ret_success
}

check_backup_install_result()
{
    if [ ${target} == "patch" ]; then
        patch_verison=$($sshpass_cmd $ssh_options root@$peer_ha_interface_ip cat ${version_path}/PATCH-${patch_target_type}-VERSION)
        package_version=${target_name##*-}
        if [[ "$package_version" == "$patch_verison" ]] ; then
            return $ret_success
        fi
        return $ret_fail_install_rpm
    elif [ ${target} == "version" ]; then
        rpm_verison=$($sshpass_cmd $ssh_options root@$peer_ha_interface_ip rpm -qa | grep $PRODUCT)
        rpm_package_name=${target_name%.*}
        if [[ "$rpm_package_name" == "$rpm_verison" ]] ; then
            return $ret_success
        fi
        return $ret_fail_install_rpm
    else
        return $ret_fail_install_rpm
    fi
}


#write_upgrade_result_conf $1=success/failure
write_upgrade_result_conf()
{
    rm $bak_conf_file -f
    cp $conf_file $bak_conf_file -f
    if [ $? -eq 0 ];then
        sed -i "/upgraderesult/d" $bak_conf_file
        sed -i "/upgradeID/d" $bak_conf_file
        echo "upgraderesult=$1"  >> $bak_conf_file
        echo "upgradeID=${result_id}"  >> $bak_conf_file
        cp $bak_conf_file $conf_file -f
    fi
}

#upgrade_product_version $1=rpm_name
upgrade_product_version()
{
    local rpm_name=$1
    log "uninstall $PRODUCT start"
    rpm -e $PRODUCT -v --noscripts --nodeps >> $logfile 2>&1
    ret=$?
    if [[ $ret -eq 0 ]]; then
        log "uninstall success"
    else
        log "uninstall failed"
        return $ret
    fi
    log "install $PRODUCT $1 start"
    rpm -i $rpm_name --nodeps >> $logfile 2>&1
    ret=$?
    if [[ $ret -eq 0 ]]; then
        log "install success"
    else
        log "install failed"
    fi
    return $ret
}


upgrade_product_version_rollback()
{
    log "UPGRADE version failed, try ROLLBACK"
    write_upgrade_result_conf "failure"
    rpm=`rpm -qa | grep $PRODUCT`
    if [ -z $rpm ];then
        for Line in $(ls -lt $upgrade_path | grep $PRODUCT | awk '{print $9}')
        do
            rpm -i $upgrade_path/$Line --nodeps >> $logfile
            rpm=`rpm -qa | grep $PRODUCT`
            if [ -z $rpm ];then
                log "install $Line failed , try again"
            else
                log "ROLLBACK to $Line success"
                log "sytemctl restart $PRODUCT"
                systemctl restart $PRODUCT
                exit
            fi
        done
        log "ROLLBACK failed , anyway exit"
        exit
    else
        log "sytemctl restart $PRODUCT"
        systemctl restart $PRODUCT
        exit 
    fi
}

upgrade_version()
{
    log "----------------$(date)--------------------"
    log "Version upgrade, target=${target_name} result_id=${result_id}"
    if [ ! -f ${upgrade_path}/${target_name} ];then
        log "Could't find $target_name in directory ${upgrade_path}, upgrade ${result_id} failed"
        cpeagent upgradeFailed
        exit
    fi

    get_HA_status "local"
    ret=$?
    if [ $ret -ne 0 ]; then
        log "$print_prefix get local HA status error, ret=$ret"
        cpeagent upgradeFailed
        exit
    fi
    log "$print_prefix local ha status ok, init_role:${init_ha_role[0]} enable:${ha_enable_state[0]} vrrp_state:${vrrp_state[0]}"

    # if ha is enable
    if [ ${init_ha_role[0]} -ne 0 ]; then
        get_valid_ha_interface
        if [ $? -ne 0 ]; then
            log "$print_prefix can not get a valid ha interface"
            cpeagent upgradeFailed
            exit
        fi
        log "$print_prefix ha interface ok, interface:$current_ha_interface peer ip:$peer_ha_interface_ip is available"

        get_HA_status "peer"
        ret=$?
        if [ $ret -ne 0 ]; then
            log "$print_prefix get peer HA status error $ret"
            cpeagent upgradeFailed
            exit
        fi
        log "$print_prefix peer ha status ok, init_role:${init_ha_role[1]} enable:${ha_enable_state[1]} vrrp_state:${vrrp_state[1]}"

        check_HA_vrrp_status
        if [ $? -ne 0 ]; then
            log "$print_prefix vrrp state failed, local is ${vrrp_state[0]}, peer is ${vrrp_state[1]}, please check it first"
            cpeagent upgradeFailed
            exit
        fi
        log "$print_prefix ha status check ok, local is ${vrrp_state[0]}, peer is ${vrrp_state[1]}"

        # if local is MASTER, need send rpm to backup and check install result
        if [ ${vrrp_state[0]} -eq 1 ]; then
            log "$print_prefix local is Matser, start to send rpm to peer for installing first"
            send_rpm_to_backup_install
            if [ $? -ne 0 ]; then
                log "$print_prefix local is Matser, send rpm to peer for install error"
                cpeagent upgradeFailed
                exit
            fi
            log "$print_prefix local is Matser, send rpm to peer ok"

            for ((i = 1; i <= $upgrade_result_query_count; i++ )); do
                wait_time_var="upgrade_result_query_time${i}_wait"
                sleep ${!wait_time_var}
                check_backup_install_result
                if [ $? -ne 0 ]; then
                    log "$print_prefix local is Matser, $i time check peer install error"
                else
                    log "$print_prefix local is Matser, check peer install ok"
                    break
                fi
            done

            if [ $i -eq $((upgrade_result_query_count + 1)) ] ; then 
                log "$print_prefix local is Matser, check peer install error"
                cpeagent upgradeFailed
                exit
            fi
        # local is backup, 
        elif [ ${vrrp_state[0]} -eq 2 ]; then
            log "$print_prefix local is backup, prepare to install rpm"
        fi
    else
        log "$print_prefix ha is disable, upgrade as a single device"
    fi

    log "$print_prefix start to upgrade rpm"
    # this is common update version logic
    upgrade_product_version "${upgrade_path}/${target_name}"

    if [[ $? -eq 0 ]];then
        log "Upgrade ${target_name} upgradeId ${result_id} success"
        write_upgrade_result_conf "success"
        log "systemctl restart $PRODUCT"
        systemctl restart $PRODUCT
        exit
    else
        upgrade_product_version_rollback
        exit
    fi
}

manual_version_upgrade()
{
    log "----------------$(date)--------------------"
    log "Manual Version upgrade"
    if [[ ! -d $upgrade_path ]];then
        echo "$upgrade_path dir does not exit"
        log "$upgrade_path dir does not exit"
        exit
    fi
    
    target_name=`ls -lt $upgrade_path | grep $PRODUCT | head -n1 | awk '{print $9}'`
    if [[ -z $target_name ]];then
        echo "Could not find any upgrade rpm in $upgrade_path"
        log "Could not find any upgrade rpm in $upgrade_path"
        exit
    fi

    upgrade_product_version "${upgrade_path}/${target_name}"
    if [[ $? -eq 0 ]]; then
        log "Manual Version upgrade success"
        log "systemctl restart $PRODUCT"
        systemctl restart $PRODUCT
    else
        upgrade_product_version_rollback
    fi
}

#handel_patch_return $1=ret
handel_patch_return()
{
    log "Upgrade ${target_name} upgradeId ${result_id} success"
    echo ${target_name##*-} > "${version_path}/PATCH-${patch_target_type}-VERSION"
    local value=$1
    if [ $value -eq 0 ];then
        cpeagent upgradeSuccess
    elif [ $value -eq ${ret_success_restart} ];then
        write_upgrade_result_conf "success"
        log "systemctl restart $PRODUCT"
        systemctl restart $PRODUCT
    elif [ $value -eq ${ret_success_reboot} ];then
        write_upgrade_result_conf "success"
        log "reboot device"
        reboot
    fi
}

#upgrade_product_patch $1=target script
upgrade_product_patch()
{
    local script=$1
    log "install ${target_name} start"
    sh ${script} install >> $logfile 2>&1
    ret=$?
    if [[ $ret -lt $ret_fail_roolback ]]; then
        log "install success $ret"
    else
        log "install failed $ret"
    fi
    return $ret
}

#upgrade_product_patch_rollback $1=target script
patch_rollback()
{
    local script=$1
    log "UPGRADE patch failed, try ROLLBACK"
    log "uninstall ${target_name}"
    sh ${script} uninstall >> $logfile 2>&1
}

get_patch_target_type()
{
    if [[ $target_name =~ THIRD ]];then
        patch_target_type="THIRD"
    elif [[ $target_name =~ CUSTOM ]];then
        patch_target_type="CUSTOM"
    elif [[ $target_name =~ DRIVER ]];then
        patch_target_type="DRIVER"
    fi
}

upgrade_patch()
{
    log "----------------$(date)--------------------"
    log "Patch upgrade, target=${target_name} result_id=${result_id}"

    get_patch_target_type
    if [ -z $patch_target_type ];then
        log "Invalid patch, exit"
        cpeagent upgradeFailed
        exit
    fi

    if [[ ! -d ${upgrade_path}/${target_name} || ! -f ${upgrade_path}/${target_name}/${target_name}.sh ]];then
        log "Could't find $target_name in directory ${upgrade_path}/${target_name}, upgrade ${result_id} failed"
        cpeagent upgradeFailed
    fi

    if [ ! -x "${upgrade_path}/${target_name}/${target_name}.sh" ];then
        chmod 751 ${upgrade_path}/${target_name}/${target_name}.sh
    fi

    # do not send to backup if patch is PATCH-X86-CUSTOM-1.0.2 or PATCH-LW2508-CUSTOM-1.0.3 because lack of jq and sshpass
    if [[ ${target} != "PATCH-X86-CUSTOM-1.0.2" && ${target} != "PATCH-LW2508-CUSTOM-1.0.3" ]];then
        get_HA_status "local"
        ret=$?
        if [ $ret -ne 0 ]; then
            log "$print_prefix get local HA status error, ret=$ret"
            cpeagent upgradeFailed
            exit
        fi
        log "$print_prefix local ha status ok, init_role:${init_ha_role[0]} enable:${ha_enable_state[0]} vrrp_state:${vrrp_state[0]}"

        # if ha is enable
        if [ ${init_ha_role[0]} -ne 0 ]; then
            get_valid_ha_interface
            if [ $? -ne 0 ]; then
                log "$print_prefix can not get a valid ha interface"
                cpeagent upgradeFailed
                exit
            fi
            log "$print_prefix ha interface ok, interface:$current_ha_interface peer ip:$peer_ha_interface_ip is available"

            get_HA_status "peer"
            ret=$?
            if [ $ret -ne 0 ]; then
                log "$print_prefix get peer HA status error $ret"
                cpeagent upgradeFailed
                exit
            fi
            log "$print_prefix peer ha status ok, init_role:${init_ha_role[1]} enable:${ha_enable_state[1]} vrrp_state:${vrrp_state[1]}"

            check_HA_vrrp_status
            if [ $? -ne 0 ]; then
                log "$print_prefix vrrp state failed, local is ${vrrp_state[0]}, peer is ${vrrp_state[1]}, please check it first"
                cpeagent upgradeFailed
                exit
            fi
            log "$print_prefix ha status check ok, local is ${vrrp_state[0]}, peer is ${vrrp_state[1]}"

            # if local is MASTER, need send patch to backup and check install result
            if [ ${vrrp_state[0]} -eq 1 ]; then
                log "$print_prefix local is Matser, start to send patch to peer for installing first"
                send_patch_to_backup_install
                if [ $? -ne 0 ]; then
                    log "$print_prefix local is Matser, send patch to peer for install error"
                    cpeagent upgradeFailed
                    exit
                fi
                log "$print_prefix local is Matser, send patch to peer ok"

                for ((i = 1; i <= $upgrade_result_query_count; i++ )); do
                    wait_time_var="upgrade_result_query_time${i}_wait"
                    sleep ${!wait_time_var}
                    check_backup_install_result
                    if [ $? -ne 0 ]; then
                        log "$print_prefix local is Matser, $i time check peer install error"
                    else
                        log "$print_prefix local is Matser, check peer install ok"
                        break
                    fi
                done

                if [ $i -eq $((upgrade_result_query_count + 1)) ] ; then 
                    log "$print_prefix local is Matser, check peer install error"
                    cpeagent upgradeFailed
                    exit
                fi
            # local is backup, 
            elif [ ${vrrp_state[0]} -eq 2 ]; then
                log "$print_prefix local is backup, prepare to install patch"
            fi
        else
            log "$print_prefix ha is disable, upgrade as a single device"
        fi
    fi

    # this is common update patch logic
    upgrade_product_patch "${upgrade_path}/${target_name}/${target_name}.sh"

    ret=$?
    if [[ $ret -lt $ret_fail_roolback ]];then
        handel_patch_return $ret
    else
        log "Upgrade ${target_name} upgradeId ${result_id} failed"
        cpeagent upgradeFailed
        if [ $ret == $ret_fail_roolback ];then
            patch_rollback "${upgrade_path}/${target_name}/${target_name}.sh"
        fi
    fi
}

help()
{
    echo "Usage: $0 --target=<patch/version> --target_name=CPE-3.5.1-20211112.x86_64.rpm --result_id=xxxx"
    echo "      --target/-t         : Target  patch/version"
    echo "      --target_name/-tn   : Upgrade target name, e.g CPE-3.5.1-20211112.x86_64.rpm"
    echo "      --result_id/-r      : upgrade Id"
}

for opt in ${@};
do
    str="="
    result=$(echo $opt | grep "$str")
    if [ -n $result ]; then
        k=${opt%=*}
        v=${opt#*=}
    else
        k=${opt}
    fi

    case ${k} in
        "--target"|"-t") target=${v};;
        "--target_name"|"-tn") target_name=${v};;
        "--result_id"|"-r") result_id=${v};;
        *) echo "${opt} is error";;
    esac
done

if [ $# -lt 3 ];then
    target="manual_version"
else
    [[ -z $target || -z $target_name || -z $result_id ]] && {
        help
        exit 1
    }
fi

if [ ${target} == "patch" ]; then
    upgrade_patch
elif [ ${target} == "version" ]; then
    upgrade_version
elif [ ${target} == "manual_version" ]; then
    echo "manual_version"
    manual_version_upgrade
else
    log "----------------$(date)--------------------"
    log "Unsupported target ${target} result_id=${result_id}"
    cpeagent upgradeFailed
fi